/*
 * Keyword Table API Test.
 *
 * Compile with:
 *	gcc -g3 -Wall -O0 -I../include -I../vmlang -L../lib \
 *	 -o keyword_test keyword_test.c ../vmlang/keyword.o -llib
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "macros.h"
#include "common.h"
#include "keyword.h"
#include "mlex.h"

#define ERROR(msg, arg...) \
do { \
	fprintf(stderr, "keyword_test: " msg "\n", ##arg); \
} while (0);

static struct keyword keywords[] = {
	{ "int",	TOKEN_DECL_INT },
	{ "float",	TOKEN_DECL_REAL },
	{ "char",	TOKEN_DECL_CHAR },
	{ "string",	TOKEN_DECL_STRING },
	{ "void",	TOKEN_DECL_VOID },
	{ "module",	TOKEN_MODULE},
	{ "if",		TOKEN_IF },
	{ "else",	TOKEN_ELSE },
	{ "for",	TOKEN_FOR },
	{ "while",	TOKEN_WHILE },
	{ "return",	TOKEN_RETURN },
	{ "break",	TOKEN_BREAK },
	{ "continue",	TOKEN_CONTINUE },
	{ NULL, -1 }
};

int
keyword_test()
{
	struct keyword *kw, *kwp;
	char *nonex[] = { "girls", "beef", "water", "world", "peace", NULL };
	char **c;
	int i;

	for (i = 0; i < 2; i++) {

		keyword_table_create();

		for (kw = keywords; kw->name != NULL; kw++) {
			if ((kwp = keyword_table_lookup(kw->name)) == NULL) {
				ERROR("keyword `%s' not found in the table", kw->name);
				goto error;
			}
			if (kwp->type != kw->type) {
				ERROR("keyword `%s' has invalid token type", kw->name);
				goto error;
			}
		}

		for (c = nonex; *c != NULL; c++) {
			if (keyword_table_lookup(*c) != NULL) {
				ERROR("nonexistent word `%s' found in the table", *c);
				goto error;
			}
		}

		keyword_table_destroy();
	}

	return 0;
error:
	return 1;
}

int
main(int argc, char *argv[])
{
	int res;
	char *msg[] = { "ok", "failed" };

	res = keyword_test();

	fprintf(stderr, "keyword_test: %s\n", msg[res]);

	return res;
}
